////////////////////////////// TP2 File d'attente //////////////////////////

////////////////////////////////////////////////////////////////////////
// créé le 13/03/2012      mis à jour le 15/03/2012
//
// Simulation d'une file d'attente MMn
//
// Descriptif des paramètres
//   p   nombre de serveurs
//   m   longueur maximum de la file d'attente
//   la   taux d'arrivée des clients
//   mu   taux de service
//   N   nombre d'événements simulés
//
// Variables internes
//   n   étiquette de l'événement courant
//   i,j étiquette du serveur (0 pour la file)
//   T array(1,N+1) dates d'occurence des événements
//   L array(1,N+1) longueurs de la file d'attente à T(n)+0
//   H array(1,N+1) nombre de clients éconduits à Tn+
//   U array(p+1,N+1) dates d'arrivée et de fin de service i suivant Tn+
//   V array(p+1,N+1) intertemps d'arrivée et de service
//   X array(p,N+1) état des serveurs suivant Tn+
//   S array(1,N+1) nombre de serveurs occupés à Tn+
//
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////
//   Fixation des paramètres de la simulation    //
///////////////////////////////////////////////////

p=10;
m=15     // %inf
la=7;
mu=1;
N=1000;
Nin=990; // visibilité des neregistrements régime stationnaire

////////////////////////////////////////////////////////
//   Initialisation des variables de la simulation    //
////////////////////////////////////////////////////////

n=0;
T=zeros(1,N+1);
L=zeros(1,N+1);
H=zeros(1,N+1);
V=zeros(p+1,N+1);
V(1,:)=grand(1,N+1,'exp',1/la)
V(2:p+1,:)=grand(p,N+1,'exp',1/mu);
U=zeros(p+1,N+1);
U(1,1)=V(1,1);
U(2:p+1,1)=%inf;
X=zeros(p+1,N+1);
S=sum(X,1);

/////////////////////
//   Iteration    //
////////////////////
for n=1:N
    L(n+1)=L(n);            // Transmission du passé
    H(n+1)=H(n);            // Transmission du passé
    X(:,n+1)=X(:,n);        // Transmission du passé
    S=sum(X,1);             // Mise à jour de S
    U(:,n+1)=U(:,n);        // Transmission du passé
    [T(1,n+1),i]=min(U(:,n));// Datation et identification de l'événement
    if i==1                 // Gestion d'une arrivée
        U(1,n+1)=T(1,n+1)+V(1,n+1);
        if S(n)==p
            if L(n)<m L(n+1)=L(n)+1;
                if n>Nin disp("Date arrivée:"+string(T(1,n+1))+",  File d''attente:"+string(L(n+1))); end;
            else H(n+1)=H(n)+1;
                if n>Nin disp("Date arrivée:"+string(T(1,n+1))+"  Client écarté"); end;
             end
        else [XX,j]=min(X(:,n)); // Identification du serveur impacté
            X(j,n+1)=1;
            S(n+1)=S(n)+1;
            U(j+1,n+1)=T(1,n+1)+V(j+1,n+1);
             if n>Nin disp("Date arrivée:"+string(T(1,n+1))+",  Affectation au serveur "+string(j)); end;
        end
    else                    // Gestion d'une fin de service de i-1
        if L(n)>0          
            L(n+1)=L(n)-1;  // Affectation en i-1 d'un client en attente
            U(i,n+1)=T(1,n+1)+V(i,n+1);
            if n>Nin disp("Date Nouvelle affectation serveur "+string(i-1)+":"+string(T(1,n+1))+",  File d''attente:"+string(L(n+1))); end;   
        else
            X(i-1,n+1)=0;
            S(n+1)=S(n)-1;
            U(i,n+1)=%inf;
            if n>Nin disp("Date fin de service serveur "+string(i-1)+":"+string(T(1,n+1))+" Disponibilité");end;
        end
    end 
end
disp("Fin du processus: Nombre de clients econduits:"+string(H(1,N+1)))

//////////////////////////////////
//   Graphes de la simulation   //
//////////////////////////////////

scf(1);
clf
plot(T,L,'-k');
xtitle("Longueur file d''attente","Date d''arrivée","Longueur");

scf(2);
clf
plot(T,S,'-k')
xtitle("Nombre de serveurs","Date d''arrivée","Nombre de serveurs occuppés");

